#
# This file is part of Redis
#
# This software is Copyright (c) 2015 by Pedro Melo, Damien Krotkine.
#
# This is free software, licensed under:
#
#   The Artistic License 2.0 (GPL Compatible)
#
# This Build.PL for Redis was generated by Dist::Zilla::Plugin::ModuleBuildTiny 0.015.
use strict;
use warnings;

use 5.006;
use Module::Build::Tiny 0.034;
Build_PL();
